@@ -15,6 +15,7 @@ import atexit
 from collections import OrderedDict
 import signal
 import sys
+import threading
 import time
 
 from enlighten._counter import Counter
@@ -36,6 +37,8 @@ class Manager(object):
             below. (Default: :py:data:`None`)
         enabled(bool): Status (Default: True)
         no_resize(bool): Disable resizing support
+        threaded(bool): When True resize handling is deferred until next write (Default: False
+            unless multiple threads are detected)
         kwargs(Dict[str, Any]): Any additional :py:term:`keyword arguments<keyword argument>`
             will be used as default values when :py:meth:`counter` is called.
 
@@ -70,6 +73,7 @@ class Manager(object):
         self.enabled = kwargs.get('enabled', True)  # Double duty for counters
         self.no_resize = kwargs.pop('no_resize', False)
         self.set_scroll = kwargs.pop('set_scroll', True)
+        self.threaded = kwargs.pop('threaded', threading.active_count() > 1)
         self.term = Terminal(stream=self.stream)
 
         # Set up companion stream
@@ -98,6 +102,7 @@ class Manager(object):
         self.height = self.term.height
         self.process_exit = False
         self.refresh_lock = False
+        self._resize = False
         self.resize_lock = False
         self.scroll_offset = 1
         self.width = self.term.width
@@ -250,14 +255,31 @@ class Manager(object):
 
         self._set_scroll_area()
         for ctr in reversed(toRefresh):
-            ctr.refresh()
-        self.stream.flush()
+            ctr.refresh(flush=False)
+        self._flush_streams()
 
         return new
 
-    def _resize_handler(self, *args, **kwarg):  # pylint: disable=unused-argument
+    def _stage_resize(self, *args, **kwarg):  # pylint: disable=unused-argument
         """
         Called when a window resize signal is detected
+        """
+
+        # Set semaphore to trigger resize on next write
+        self._resize = True
+
+        if self.threaded:
+            # Reset update time to avoid any delay in resize
+            for counter in self.counters:
+                counter.last_update = 0
+
+        else:
+            # If not threaded, handle resize now
+            self._resize_handler()
+
+    def _resize_handler(self):
+        """
+        Called when a window resize has been detected
 
         Resets the scroll window
         """
@@ -273,27 +295,23 @@ class Manager(object):
             term.clear_cache()
             newHeight = term.height
             newWidth = term.width
-            lastHeight = lastWidth = 0
 
-            while newHeight != lastHeight or newWidth != lastWidth:
-                lastHeight = newHeight
-                lastWidth = newWidth
-                time.sleep(.2)
-                term.clear_cache()
-                newHeight = term.height
-                newWidth = term.width
+            if newHeight < self.height:
+                term.move_to(0, max(0, newHeight - self.scroll_offset))
+                self.stream.write(u'\n' * (2 * max(self.counters.values())))
+            elif newHeight > self.height and self.threaded:
+                term.move_to(0, newHeight)
+                self.stream.write(u'\n' * (self.scroll_offset - 1))
 
-            if newWidth < self.width:
-                offset = (self.scroll_offset - 1) * (1 + self.width // newWidth)
-                term.move_to(0, max(0, newHeight - offset))
-                self.stream.write(term.clear_eos)
+            term.move_to(0, max(0, newHeight - self.scroll_offset))
+            self.stream.write(term.clear_eos)
 
             self.width = newWidth
             self._set_scroll_area(force=True)
 
             for counter in self.counters:
                 counter.refresh(flush=False)
-            self.stream.flush()
+            self._flush_streams()
 
             self.resize_lock = False
 
@@ -321,7 +339,7 @@ class Manager(object):
         if not self.process_exit:
             atexit.register(self._at_exit)
             if not self.no_resize and RESIZE_SUPPORTED:
-                signal.signal(signal.SIGWINCH, self._resize_handler)
+                signal.signal(signal.SIGWINCH, self._stage_resize)
             self.process_exit = True
 
         if self.set_scroll:
@@ -346,6 +364,15 @@ class Manager(object):
             if self.companion_term is not None:
                 self.companion_term.move_to(0, scrollPosition)
 
+    def _flush_streams(self):
+        """
+        Convenience method for flushing streams
+        """
+
+        self.stream.flush()
+        if self.companion_stream is not None:
+            self.companion_stream.flush()
+
     def _at_exit(self):
         """
         Resets terminal to normal configuration
@@ -364,9 +391,7 @@ class Manager(object):
 
             self.term.feed()
 
-            self.stream.flush()
-            if self.companion_stream is not None:
-                self.companion_stream.flush()
+            self._flush_streams()
 
         except ValueError:  # Possibly closed file handles
             pass
@@ -420,8 +445,6 @@ class Manager(object):
                     term.move_to(0, term.height - num)
                     stream.write(term.clear_eol)
 
-            stream.flush()
-
         finally:
 
             if self.set_scroll:
@@ -443,27 +466,47 @@ class Manager(object):
         if 1 in positions:
             term.feed()
 
-    def write(self, output='', flush=True, counter=None):
+        self._flush_streams()
+
+    def write(self, output='', flush=True, counter=None, **kwargs):
         """
         Args:
-            output(str): Output string
+            output(str): Output string or callable
             flush(bool): Flush the output stream after writing
             counter(:py:class:`Counter`): Bar being written (for position and auto-refresh)
+            kwargs(dict): Additional arguments passed when output is callable
 
-        Write to stream at a given position
+        Write to the stream.
+
+        The position is determined by the counter or defaults to the bottom of the terminal
+
+        If ``output`` is callable, it will be called with any additional keyword arguments
+        to produce the output string
         """
 
         if not self.enabled:
             return
 
+        # If resize signal was caught, handle resize
+        if self._resize and not self.resize_lock:
+            try:
+                self._resize_handler()
+            finally:
+                self._resize = False
+
+            return
+
         position = self.counters[counter] if counter else 0
-        stream = self.stream
         term = self.term
 
+        # If output is callable, call it with supplied arguments
+        if callable(output):
+            output = output(**kwargs)
+
         try:
             term.move_to(0, term.height - position)
             # Include \r and term call to cover most conditions
-            stream.write(u'\r' + term.clear_eol + output)
+            self.stream.write(u'\r' + term.clear_eol + output)
 
         finally:
             # Reset position and scrolling
@@ -472,7 +515,7 @@ class Manager(object):
                     self._autorefresh(exclude=(counter,))
                 self._set_scroll_area()
                 if flush:
-                    stream.flush()
+                    self._flush_streams()
 
     def _autorefresh(self, exclude):
         """
